@@ -4,27 +4,59 @@
 #
 # This software may be used and distributed according to the terms
 # of the GNU General Public License, incorporated herein by reference.
+#
+# The hgk extension allows browsing the history of a repository in a
+# graphical way. It requires Tcl/Tk version 8.4 or later. (Tcl/Tk is
+# not distributed with Mercurial.)
+#
+# hgk consists of two parts: a Tcl script that does the displaying and
+# querying of information, and an extension to mercurial named hgk.py,
+# which provides hooks for hgk to get information. hgk can be found in
+# the contrib directory, and hgk.py can be found in the hgext
+# directory.  
+#
+# To load the hgext.py extension, add it to your .hgrc file (you have
+# to use your global $HOME/.hgrc file, not one in a repository). You
+# can specify an absolute path:
+#
+#   [extensions]
+#   hgk=/usr/local/lib/hgk.py
+#
+# Mercurial can also scan the default python library path for a file
+# named 'hgk.py' if you set hgk empty:
+#
+#   [extensions]
+#   hgk=
+#
+# The hg view command will launch the hgk Tcl script. For this command
+# to work, hgk must be in your search path. Alternately, you can
+# specify the path to hgk in your .hgrc file:
+#
+#   [hgk]
+#   path=/location/of/hgk
+#
+# hgk can make use of the extdiff extension to visualize
+# revisions. Assuming you had already configured extdiff vdiff
+# command, just add:
+#
+#   [hgk]
+#   vdiff=vdiff
+#
+# Revisions context menu will now display additional entries to fire
+# vdiff on hovered and selected revisions.
 
-from mercurial.demandload import *
-demandload(globals(), 'time sys signal os')
-demandload(globals(), 'mercurial:hg,fancyopts,commands,ui,util,patch,revlog')
+import sys, os
+from mercurial import hg, fancyopts, commands, ui, util, patch, revlog
 
 def difftree(ui, repo, node1=None, node2=None, *files, **opts):
     """diff trees from two commits"""
     def __difftree(repo, node1, node2, files=[]):
-        if node2:
-            change = repo.changelog.read(node2)
-            mmap2 = repo.manifest.read(change[0])
-            status = repo.status(node1, node2, files=files)[:5]
-            modified, added, removed, deleted, unknown = status
-        else:
-            status = repo.status(node1, files=files)[:5]
-            modified, added, removed, deleted, unknown = status
-            if not node1:
-                node1 = repo.dirstate.parents()[0]
+        assert node2 is not None
+        mmap = repo.changectx(node1).manifest()
+        mmap2 = repo.changectx(node2).manifest()
+        status = repo.status(node1, node2, files=files)[:5]
+        modified, added, removed, deleted, unknown = status
 
-        change = repo.changelog.read(node1)
-        mmap = repo.manifest.read(change[0])
         empty = hg.short(hg.nullid)
 
         for f in modified:
@@ -70,32 +102,30 @@ def difftree(ui, repo, node1=None, node2=None, *files, **opts):
         if not opts['stdin']:
             break
 
-def catcommit(repo, n, prefix, changes=None):
+def catcommit(repo, n, prefix, ctx=None):
     nlprefix = '\n' + prefix;
-    (p1, p2) = repo.changelog.parents(n)
-    (h, h1, h2) = map(hg.short, (n, p1, p2))
-    (i1, i2) = map(repo.changelog.rev, (p1, p2))
-    if not changes:
-        changes = repo.changelog.read(n)
-    print "tree %s" % (hg.short(changes[0]))
-    if i1 != hg.nullrev: print "parent %s" % (h1)
-    if i2 != hg.nullrev: print "parent %s" % (h2)
-    date_ar = changes[2]
-    date = int(float(date_ar[0]))
-    lines = changes[4].splitlines()
+    if ctx is None:
+        ctx = repo.changectx(n)
+    (p1, p2) = ctx.parents()
+    print "tree %s" % (hg.short(ctx.changeset()[0])) # use ctx.node() instead ??
+    if p1: print "parent %s" % (hg.short(p1.node()))
+    if p2: print "parent %s" % (hg.short(p2.node()))
+    date = ctx.date()
+    description = ctx.description().replace("\0", "")
+    lines = description.splitlines()
     if lines and lines[-1].startswith('committer:'):
         committer = lines[-1].split(': ')[1].rstrip()
     else:
-        committer = changes[1]
+        committer = ctx.user()
 
-    print "author %s %s %s" % (changes[1], date, date_ar[1])
-    print "committer %s %s %s" % (committer, date, date_ar[1])
-    print "revision %d" % repo.changelog.rev(n)
+    print "author %s %s %s" % (ctx.user(), int(date[0]), date[1])
+    print "committer %s %s %s" % (committer, int(date[0]), date[1])
+    print "revision %d" % ctx.rev()
     print ""
     if prefix != "":
-        print "%s%s" % (prefix, changes[4].replace('\n', nlprefix).strip())
+        print "%s%s" % (prefix, description.replace('\n', nlprefix).strip())
     else:
-        print changes[4]
+        print description
     if prefix:
         sys.stdout.write('\0')
 
@@ -146,8 +176,7 @@ def catfile(ui, repo, type=None, r=None, **opts):
 # you can specify a commit to stop at by starting the sha1 with ^
 def revtree(args, repo, full="tree", maxnr=0, parents=False):
     def chlogwalk():
-        ch = repo.changelog
-        count = ch.count()
+        count = repo.changelog.count()
         i = count
         l = [0] * 100
         chunk = 100
@@ -163,7 +192,8 @@ def revtree(args, repo, full="tree", maxnr=0, parents=False):
                     l[chunk - x:] = [0] * (chunk - x)
                     break
                 if full != None:
-                    l[x] = ch.read(ch.node(i + x))
+                    l[x] = repo.changectx(i + x)
+                    l[x].changeset() # force reading
                 else:
                     l[x] = 1
             for x in xrange(chunk-1, -1, -1):
@@ -217,7 +247,7 @@ def revtree(args, repo, full="tree", maxnr=0, parents=False):
 
     # walk the repository looking for commits that are in our
     # reachability graph
-    for i, changes in chlogwalk():
+    for i, ctx in chlogwalk():
         n = repo.changelog.node(i)
         mask = is_reachable(want_sha1, reachable, n)
         if mask:
@@ -232,13 +262,13 @@ def revtree(args, repo, full="tree", maxnr=0, parents=False):
                 print hg.short(n) + parentstr
             elif full == "commit":
                 print hg.short(n) + parentstr
-                catcommit(repo, n, '    ', changes)
+                catcommit(repo, n, '    ', ctx)
             else:
                 (p1, p2) = repo.changelog.parents(n)
                 (h, h1, h2) = map(hg.short, (n, p1, p2))
                 (i1, i2) = map(repo.changelog.rev, (p1, p2))
 
-                date = changes[2][0]
+                date = ctx.date()[0]
                 print "%s %s:%s" % (date, h, mask),
                 mask = is_reachable(want_sha1, reachable, p1)
                 if i1 != hg.nullrev and mask > 0:
@@ -276,34 +306,53 @@ def revlist(ui, repo, *revs, **opts):
     copy = [x for x in revs]
     revtree(copy, repo, full, opts['max_count'], opts['parents'])
 
+def config(ui, repo, **opts):
+    """print extension options"""
+    def writeopt(name, value):
+        ui.write('k=%s\nv=%s\n' % (name, value)) 
+
+    writeopt('vdiff', ui.config('hgk', 'vdiff', ''))
+    
+
 def view(ui, repo, *etc, **opts):
     "start interactive history viewer"
     os.chdir(repo.root)
     optstr = ' '.join(['--%s %s' % (k, v) for k, v in opts.iteritems() if v])
     cmd = ui.config("hgk", "path", "hgk") + " %s %s" % (optstr, " ".join(etc))
     ui.debug("running %s\n" % cmd)
-    os.system(cmd)
+    util.system(cmd)
 
 cmdtable = {
-    "^view": (view,
-             [('l', 'limit', '', 'limit number of changes displayed')],
-             'hg view [-l LIMIT] [REVRANGE]'),
-    "debug-diff-tree": (difftree, [('p', 'patch', None, 'generate patch'),
-                            ('r', 'recursive', None, 'recursive'),
-                            ('P', 'pretty', None, 'pretty'),
-                            ('s', 'stdin', None, 'stdin'),
-                            ('C', 'copy', None, 'detect copies'),
-                            ('S', 'search', "", 'search')],
-                            "hg git-diff-tree [options] node1 node2 [files...]"),
-    "debug-cat-file": (catfile, [('s', 'stdin', None, 'stdin')],
-                 "hg debug-cat-file [options] type file"),
-    "debug-merge-base": (base, [], "hg debug-merge-base node node"),
-    'debug-rev-parse': (revparse,
-                        [('', 'default', '', 'ignored')],
-                        "hg debug-rev-parse rev"),
-    "debug-rev-list": (revlist, [('H', 'header', None, 'header'),
-                           ('t', 'topo-order', None, 'topo-order'),
-                           ('p', 'parents', None, 'parents'),
-                           ('n', 'max-count', 0, 'max-count')],
-                 "hg debug-rev-list [options] revs"),
+    "^view":
+        (view,
+         [('l', 'limit', '', 'limit number of changes displayed')],
+         'hg view [-l LIMIT] [REVRANGE]'),
+    "debug-diff-tree":
+        (difftree,
+         [('p', 'patch', None, 'generate patch'),
+          ('r', 'recursive', None, 'recursive'),
+          ('P', 'pretty', None, 'pretty'),
+          ('s', 'stdin', None, 'stdin'),
+          ('C', 'copy', None, 'detect copies'),
+          ('S', 'search', "", 'search')],
+         'hg git-diff-tree [OPTION]... NODE1 NODE2 [FILE]...'),
+    "debug-cat-file":
+        (catfile,
+         [('s', 'stdin', None, 'stdin')],
+         'hg debug-cat-file [OPTION]... TYPE FILE'),
+    "debug-config":
+        (config, [], 'hg debug-config'),
+    "debug-merge-base":
+        (base, [], 'hg debug-merge-base node node'),
+    "debug-rev-parse":
+        (revparse,
+         [('', 'default', '', 'ignored')],
+         'hg debug-rev-parse REV'),
+    "debug-rev-list":
+        (revlist,
+         [('H', 'header', None, 'header'),
+          ('t', 'topo-order', None, 'topo-order'),
+          ('p', 'parents', None, 'parents'),
+          ('n', 'max-count', 0, 'max-count')],
+         'hg debug-rev-list [options] revs'),
 }
