@@ -40,6 +40,7 @@
 #   changegroup = ...      # template when run as changegroup hook
 #   maxdiff = 300          # max lines of diffs to include (0=none, -1=all)
 #   maxsubject = 67        # truncate subject line longer than this
+#   diffstat = True        # add a diffstat before the diff content
 #   sources = serve        # notify if source of incoming changes in this list
 #                          # (serve == ssh or http, push, pull, bundle)
 #   [email]
@@ -64,11 +65,10 @@
 # if you like, you can put notify config file in repo that users can
 # push changes to, they can manage their own subscriptions.
 
-from mercurial.demandload import *
-from mercurial.i18n import gettext as _
+from mercurial.i18n import _
 from mercurial.node import *
-demandload(globals(), 'email.Parser mercurial:commands,templater,util')
-demandload(globals(), 'fnmatch socket time')
+from mercurial import patch, cmdutil, templater, util, mail
+import email.Parser, fnmatch, socket, time
 
 # template for single changeset can include email headers.
 single_template = '''
@@ -92,7 +92,7 @@ summary: {desc|firstline}
 
 deftemplates = {
     'changegroup': multiple_template,
-    }
+}
 
 class notifier(object):
     '''email notification class.'''
@@ -101,19 +101,18 @@ class notifier(object):
         self.ui = ui
         cfg = self.ui.config('notify', 'config')
         if cfg:
-            self.ui.readconfig(cfg)
+            self.ui.readsections(cfg, 'usersubs', 'reposubs')
         self.repo = repo
         self.stripcount = int(self.ui.config('notify', 'strip', 0))
         self.root = self.strip(self.repo.root)
         self.domain = self.ui.config('notify', 'domain')
-        self.sio = templater.stringio()
         self.subs = self.subscribers()
 
         mapfile = self.ui.config('notify', 'style')
         template = (self.ui.config('notify', hooktype) or
                     self.ui.config('notify', 'template'))
-        self.t = templater.changeset_templater(self.ui, self.repo, mapfile,
-                                               self.sio)
+        self.t = cmdutil.changeset_templater(self.ui, self.repo,
+                                             False, mapfile, False)
         if not mapfile and not template:
             template = deftemplates.get(hooktype) or single_template
         if template:
@@ -137,11 +136,12 @@ class notifier(object):
         '''try to clean up email addresses.'''
 
         addr = templater.email(addr.strip())
-        a = addr.find('@localhost')
-        if a != -1:
-            addr = addr[:a]
-        if '@' not in addr:
-            return addr + '@' + self.domain
+        if self.domain:
+            a = addr.find('@localhost')
+            if a != -1:
+                addr = addr[:a]
+            if '@' not in addr:
+                return addr + '@' + self.domain
         return addr
 
     def subscribers(self):
@@ -176,12 +176,11 @@ class notifier(object):
         ok_sources = self.ui.config('notify', 'sources', 'serve').split()
         return source not in ok_sources
 
-    def send(self, node, count):
+    def send(self, node, count, data):
         '''send message.'''
 
         p = email.Parser.Parser()
-        self.sio.seek(0)
-        msg = p.parse(self.sio)
+        msg = p.parsestr(data)
 
         def fix_subject():
             '''try to make subject line exist and be useful.'''
@@ -211,6 +210,9 @@ class notifier(object):
             del msg['From']
             msg['From'] = sender
 
+        msg['Date'] = util.datestr(date=util.makedate(),
+                                   format="%a, %d %b %Y %H:%M:%S",
+                                   timezone=True)
         fix_subject()
         fix_sender()
 
@@ -228,25 +230,30 @@ class notifier(object):
                 self.ui.write('\n')
         else:
             self.ui.status(_('notify: sending %d subscribers %d changes\n') %
-                             (len(self.subs), count))
-            mail = self.ui.sendmail()
-            mail.sendmail(templater.email(msg['From']), self.subs, msgtext)
+                           (len(self.subs), count))
+            mail.sendmail(self.ui, templater.email(msg['From']),
+                          self.subs, msgtext)
 
     def diff(self, node, ref):
         maxdiff = int(self.ui.config('notify', 'maxdiff', 300))
         if maxdiff == 0:
             return
-        fp = templater.stringio()
         prev = self.repo.changelog.parents(node)[0]
-        commands.dodiff(fp, self.ui, self.repo, prev, ref)
-        difflines = fp.getvalue().splitlines(1)
+        self.ui.pushbuffer()
+        patch.diff(self.repo, prev, ref)
+        difflines = self.ui.popbuffer().splitlines(1)
+        if self.ui.configbool('notify', 'diffstat', True):
+            s = patch.diffstat(difflines)
+            # s may be nil, don't include the header if it is
+            if s:
+                self.ui.write('\ndiffstat:\n\n%s' % s)
         if maxdiff > 0 and len(difflines) > maxdiff:
-            self.sio.write(_('\ndiffs (truncated from %d to %d lines):\n\n') %
-                           (len(difflines), maxdiff))
+            self.ui.write(_('\ndiffs (truncated from %d to %d lines):\n\n') %
+                          (len(difflines), maxdiff))
             difflines = difflines[:maxdiff]
         elif difflines:
-            self.sio.write(_('\ndiffs (%d lines):\n\n') % len(difflines))
-        self.sio.write(*difflines)
+            self.ui.write(_('\ndiffs (%d lines):\n\n') % len(difflines))
+        self.ui.write(*difflines)
 
 def hook(ui, repo, hooktype, node=None, source=None, **kwargs):
     '''send email notifications to interested subscribers.
@@ -255,13 +262,14 @@ def hook(ui, repo, hooktype, node=None, source=None, **kwargs):
     changegroup. else send one email per changeset.'''
     n = notifier(ui, repo, hooktype)
     if not n.subs:
-        ui.debug(_('notify: no subscribers to this repo\n'))
+        ui.debug(_('notify: no subscribers to repo %s\n') % n.root)
         return
     if n.skipsource(source):
         ui.debug(_('notify: changes have source "%s" - skipping\n') %
-                  source)
+                 source)
         return
     node = bin(node)
+    ui.pushbuffer()
     if hooktype == 'changegroup':
         start = repo.changelog.rev(node)
         end = repo.changelog.count()
@@ -273,4 +281,5 @@ def hook(ui, repo, hooktype, node=None, source=None, **kwargs):
         count = 1
         n.node(node)
         n.diff(node, node)
-    n.send(node, count)
+    data = ui.popbuffer()
+    n.send(node, count, data)
