@@ -24,6 +24,16 @@
 #include <stdlib.h>
 #include <string.h>
 
+/* Definitions to get compatibility with python 2.4 and earlier which
+   does not have Py_ssize_t. See also PEP 353.
+   Note: msvc (8 or earlier) does not have ssize_t, so we use Py_ssize_t.
+*/
+#if PY_VERSION_HEX < 0x02050000 && !defined(PY_SSIZE_T_MIN)
+typedef int Py_ssize_t;
+#define PY_SSIZE_T_MAX INT_MAX
+#define PY_SSIZE_T_MIN INT_MIN
+#endif
+
 #ifdef _WIN32
 # ifdef _MSC_VER
 /* msvc 6.0 has problems */
@@ -42,7 +52,11 @@ static uint32_t ntohl(uint32_t x)
 #else
 /* not windows */
 # include <sys/types.h>
-# include <arpa/inet.h>
+# ifdef __BEOS__
+#  include <ByteOrder.h>
+# else
+#  include <arpa/inet.h>
+# endif
 # include <inttypes.h>
 #endif
 
@@ -51,7 +65,7 @@ static PyObject *mpatch_Error;
 
 struct frag {
 	int start, end, len;
-	char *data;
+	const char *data;
 };
 
 struct flist {
@@ -217,11 +231,11 @@ static struct flist *combine(struct flist *a, struct flist *b)
 }
 
 /* decode a binary patch into a hunk list */
-static struct flist *decode(char *bin, int len)
+static struct flist *decode(const char *bin, int len)
 {
 	struct flist *l;
 	struct frag *lt;
-	char *end = bin + len;
+	const char *data = bin + 12, *end = bin + len;
 	char decode[12]; /* for dealing with alignment issues */
 
 	/* assume worst case size, we won't have many of these lists */
@@ -231,13 +245,18 @@ static struct flist *decode(char *bin, int len)
 
 	lt = l->tail;
 
-	while (bin < end) {
+	while (data <= end) {
 		memcpy(decode, bin, 12);
 		lt->start = ntohl(*(uint32_t *)decode);
 		lt->end = ntohl(*(uint32_t *)(decode + 4));
 		lt->len = ntohl(*(uint32_t *)(decode + 8));
-		lt->data = bin + 12;
-		bin += 12 + lt->len;
+		if (lt->start > lt->end)
+			break; /* sanity check */
+		bin = data + lt->len;
+		if (bin < data)
+			break; /* big data + big (bogus) len can wrap around */
+		lt->data = data;
+		data = bin + 12;
 		lt++;
 	}
 
@@ -275,7 +294,7 @@ static int calcsize(int len, struct flist *l)
 	return outlen;
 }
 
-static int apply(char *buf, char *orig, int len, struct flist *l)
+static int apply(char *buf, const char *orig, int len, struct flist *l)
 {
 	struct frag *f = l->head;
 	int last = 0;
@@ -303,13 +322,17 @@ static int apply(char *buf, char *orig, int len, struct flist *l)
 static struct flist *fold(PyObject *bins, int start, int end)
 {
 	int len;
+	Py_ssize_t blen;
+	const char *buffer;
 
 	if (start + 1 == end) {
 		/* trivial case, output a decoded list */
 		PyObject *tmp = PyList_GetItem(bins, start);
 		if (!tmp)
 			return NULL;
-		return decode(PyString_AsString(tmp), PyString_Size(tmp));
+		if (PyObject_AsCharBuffer(tmp, &buffer, &blen))
+			return NULL;
+		return decode(buffer, blen);
 	}
 
 	/* divide and conquer, memory management is elsewhere */
@@ -323,10 +346,12 @@ patches(PyObject *self, PyObject *args)
 {
 	PyObject *text, *bins, *result;
 	struct flist *patch;
-	char *in, *out;
+	const char *in;
+	char *out;
 	int len, outlen;
+	Py_ssize_t inlen;
 
-	if (!PyArg_ParseTuple(args, "SO:mpatch", &text, &bins))
+	if (!PyArg_ParseTuple(args, "OO:mpatch", &text, &bins))
 		return NULL;
 
 	len = PyList_Size(bins);
@@ -336,11 +361,14 @@ patches(PyObject *self, PyObject *args)
 		return text;
 	}
 
+	if (PyObject_AsCharBuffer(text, &in, &inlen))
+		return NULL;
+
 	patch = fold(bins, 0, len);
 	if (!patch)
 		return NULL;
 
-	outlen = calcsize(PyString_Size(text), patch);
+	outlen = calcsize(inlen, patch);
 	if (outlen < 0) {
 		result = NULL;
 		goto cleanup;
@@ -350,9 +378,8 @@ patches(PyObject *self, PyObject *args)
 		result = NULL;
 		goto cleanup;
 	}
-	in = PyString_AsString(text);
 	out = PyString_AsString(result);
-	if (!apply(out, in, PyString_Size(text), patch)) {
+	if (!apply(out, in, inlen, patch)) {
 		Py_DECREF(result);
 		result = NULL;
 	}
@@ -367,20 +394,26 @@ patchedsize(PyObject *self, PyObject *args)
 {
 	long orig, start, end, len, outlen = 0, last = 0;
 	int patchlen;
-	char *bin, *binend;
+	char *bin, *binend, *data;
 	char decode[12]; /* for dealing with alignment issues */
 
 	if (!PyArg_ParseTuple(args, "ls#", &orig, &bin, &patchlen))
 		return NULL;
 
 	binend = bin + patchlen;
+	data = bin + 12;
 
-	while (bin < binend) {
+	while (data <= binend) {
 		memcpy(decode, bin, 12);
 		start = ntohl(*(uint32_t *)decode);
 		end = ntohl(*(uint32_t *)(decode + 4));
 		len = ntohl(*(uint32_t *)(decode + 8));
-		bin += 12 + len;
+		if (start > end)
+			break; /* sanity check */
+		bin = data + len;
+		if (bin < data)
+			break; /* big data + big (bogus) len can wrap around */
+		data = bin + 12;
 		outlen += start - last;
 		last = end;
 		outlen += len;
