@@ -1,6 +1,3 @@
-#!/usr/bin/env python
-#
-
 import doctest
 
 import mercurial.changelog
@@ -8,3 +5,5 @@ import mercurial.changelog
 
 doctest.testmod(mercurial.changelog)
 
+import mercurial.httprepo
+doctest.testmod(mercurial.httprepo)
